
#include <stdlib.h>
#include <stdio.h>
#include "stack.h"

/* *************************************************************** Strukturen */

typedef struct sElement
{
	void* datum;
	struct sElement* pred;
} element;

struct sStack
{
	element* top;
};

/* ******************************************************* private Funktionen */

static void terminate(const char* description)
{
	fprintf(stderr, "Error: %s\n", description);
	exit(-1);
}

/* *************************************************** exportierte Funktionen */

stack* newStack()
{
	stack* res = (stack*) calloc(1, sizeof(stack));
	
	if (!res)
		terminate("Memory limit exceeded");
	
	return res;
}

void deleteStack(stack* s)
{
	if (s)
	{
		while (!stackIsEmpty(s))
			stackPop(s);
		
		free(s);
	}
	else
		terminate("NULL pointer exception (deleteStack)");
}

void stackPush(stack* s, void* i)
{
	if (s)
	{
		element* newElement = (element*) malloc(sizeof(element));
		
		if (!newElement)
			terminate("Memory limit exceeded");

		newElement->datum = i;
		newElement->pred = s->top;
		s->top = newElement;
	}
	else
		terminate("NULL pointer exception (stackPush)");
}

void* stackTop(const stack* s)
{
	if (stackIsEmpty(s))
		terminate("Stack is empty");
	
	return s->top->datum;
}

void* stackPop(stack* s)
{
	element* tmp;
	void* res;
	
	if (stackIsEmpty(s))
		terminate("Trying to pop empty stack");
	
	tmp = s->top;
	res = tmp->datum;
	s->top = tmp->pred;
	free(tmp);
	
	return res;
}

int stackIsEmpty(const stack* s)
{
	if (!s)
		terminate("NULL pointer exception (stackIsEmpty)");
	
	return (s->top == 0);	
}

void stackPrint(const stack* s, fStackPrinter print)
{
	if (s)
	{
		element* curr = s->top;
		stack* out = newStack();
		
		while (curr)
		{
			stackPush(out, curr->datum);
			curr = curr->pred;
		}
		
		curr = out->top;
		
		while (curr)
		{
			if (print)
				print(curr->datum);
			else
				printf("%p\n", curr->datum);
			
			curr = curr->pred;
		}
		
		deleteStack(out);
	}
	else
		printf("Stack doesn't exist.");
}
