
#ifndef STRINGBUFFER_H_INCLUDED
#define STRINGBUFFER_H_INCLUDED

#include <stdio.h>

typedef struct sStringBuffer stringBuffer;

/**@brief Erstellt einen neuen Stringbuffer und gibt einen Zeiger darauf
 *        zurueck.
 */
extern stringBuffer* newStringBuffer();

/**@brief Loescht einen Stringbuffer vollstaendig.
 * @param buffer   Stringbuffer
 */
extern void deleteStringBuffer(stringBuffer* buffer);

/**@brief Haengt einen String an den Stringbuffer an.
 * @param buffer   Stringbuffer
 * @param text     String
 */
extern void stringBufferAppend(stringBuffer* buffer, const char* text);

/**@brief Fuegt einen String nach einer gegebenen Position in den Stringbuffer
 *        ein.
 * @param buffer   Stringbuffer
 * @param address  Adresse im Stringbuffer
 * @param text     String
 */
extern void stringBufferInsert(stringBuffer* buffer, const void* address,
                               const char* text);

/**@brief Gibt den Zeiger auf den aktuell letzten String zurück.
 */
extern const void* stringBufferGetCurrentAddress(const stringBuffer* buffer);

/**@brief Gibt alle gespeicherten Strings nacheinander zeilenweise aus.
 * @param buffer   Stringbuffer
 */
extern void stringBufferPrint(const stringBuffer* buffer, FILE* sOut);

/**@brief Haengt einen Stringbuffer an einen anderen.
 * @param buf1     Ziel-Stringbuffer
 * @param buf2     Quell-Stringbuffer
 * @note buf2 wird im Verlaufe dieser Operation geloescht
 */
extern void stringBufferConcatenate(stringBuffer* buf1, stringBuffer* buf2);

#endif
